import datetime
from AbstractFile import AbstractFile
import logging

##
# @brief EMWIN File
# @details Stores EMWIN file data
# @author Miguel Dominguez
# @date 2016
class EMWINFile(AbstractFile):

   CHECKSUM_MODULUS = (2**16)
   MAX_AGE_FACTOR = 5
   
   ##
   # @fn __init__
   # @brief Constructor
   # @details Initiates all relevant variables
   def __init__(self):
      self.logger = logging.getLogger(__name__)
      self.filename = ''
      self.totalBlocks = 0
      self.timestamp = datetime.datetime(datetime.MINYEAR,1,1)
      self.data = dict()
      self.age = 0
      self.currentChecksum = 0
      self.currentData = []
      self.currentBlock = 0
   
   ##
   # @fn isFileComplete
   # @brief Checks that all packets in this file have been received
   # @warning Honestly it should probably check that all the indices are
   # valid as well, but this is a quick hack to get things running
   def isFileComplete(self):
      self.logger.debug('Number of Packets in Hand {0} Total Packets {1}'.format(len(self.data),self.totalBlocks))
      return len(self.data) == self.totalBlocks

   ##
   # @fn verifyData
   # @brief Add data to the file, contingent on it being valid data
   # @details The checksum is calculated and checked for validity, and the
   # existence of the data is checked.  This is because the same packet
   # will be sent multiple times, and we don't need to store it multiple
   # times if our SNR is really great
   # @param[in] data 1024-length unsigned byte array
   # @param[in] checksum 16-bit unsigned word
   # @param[in] index The location of the packet within the file
   # @todo Verify the types and length of the data coming in!
   def verifyData(self):
      result = True
      checksumCompare = sum(self.currentData) #% self.CHECKSUM_MODULUS
      self.logger.debug('Reference Checksum: {0} Calculated Checksum: {1} Min: {2}'.format(self.currentChecksum,checksumCompare,min(self.currentData)))
      if checksumCompare == self.currentChecksum and self.currentBlock not in self.data:
         self.data[self.currentBlock] = self.currentData
      else:
         result = False
      return result

   ##
   # @fn increaseAgeAndCheck
   # @brief Increment age by one and check to see if file is "too old."
   # @details This seems like an odd setup, but there actually isn't a clear
   # way to tell when a file is permanently corrupted or incomplete.  As
   # a heuristic we just check the number of EMWIN packets that have been
   # processed without this file being finished, which is a factor of the
   # total number of packets in the file.
   def increaseAgeAndCheck(self):
      result = False      
      self.age += 1
      if self.age >= self.totalBlocks * self.MAX_AGE_FACTOR:
         result = True
      return result

   ##
   # @fn toFile
   # @brief Writes data to a file.
   # @warning Assumes the file is complete.
   def toFile(self):
      dataBuffer = []
      if self.filename:
         self.logger.debug('Present Data')
         #self.logger.debug(self.data)
         for index in sorted(self.data):
            dataBuffer += self.data[index]
         #self.logger.debug(dataBuffer)
         fileData = bytearray(dataBuffer)
         self.logger.info('Writing EMWIN file: {0}'.format(self.filename))
         with open(self.filename,'w') as fileToWrite:
            fileToWrite.write(fileData)
      else:
         self.logger.info("Could not write EMWIN file.  No file name!")

   
